@@ -49,6 +49,8 @@ enum {
 	/* The following are off by default: */
 	LXC_ATTACH_REMOUNT_PROC_SYS      = 0x00010000, /*!< Remount /proc filesystem */
 	LXC_ATTACH_LSM_NOW               = 0x00020000, /*!< FIXME: unknown */
+	/* Set PR_SET_NO_NEW_PRIVS to block execve() gainable privileges. */
+	LXC_ATTACH_NO_NEW_PRIVS		 = 0x00040000, /*!< PR_SET_NO_NEW_PRIVS */
 
 	/* We have 16 bits for things that are on by default and 16 bits that
 	 * are off by default, that should be sufficient to keep binary
@@ -143,9 +145,7 @@ typedef struct lxc_attach_options_t {
 		/* .env_policy = */     LXC_ATTACH_KEEP_ENV,                   \
 		/* .extra_env_vars = */ NULL,                                  \
 		/* .extra_keep_env = */ NULL,                                  \
-		/* .stdin_fd = */       0,                                     \
-		/* .stdout_fd = */      1,                                     \
-		/* .stderr_fd = */      2,                                     \
+		/* .stdin_fd = */       0, 1, 2                                \
 	}
 
 /*!
