@@ -523,8 +523,6 @@ static bool find_hierarchy_mountpts( struct cgroup_meta_data *meta_data, char **
 			}
 		}
 		lxc_free_array((void **)subsystems, free);
-		if (!h)
-			goto out;
 
 		r = lxc_grow_array((void ***)&meta_data->mount_points, &mount_point_capacity, mount_point_count + 1, 12);
 		if (r < 0)
@@ -557,10 +555,7 @@ static bool find_hierarchy_mountpts( struct cgroup_meta_data *meta_data, char **
 			}
 		}
 
-		if (h)
-			k = lxc_array_len((void **)h->all_mount_points);
-		else
-			k = 0;
+		k = lxc_array_len((void **)h->all_mount_points);
 		r = lxc_grow_array((void ***)&h->all_mount_points, &h->all_mount_point_capacity, k + 1, 4);
 		if (r < 0)
 			goto out;
@@ -766,7 +761,7 @@ static struct cgroup_process_info *lxc_cgroup_process_info_get_self(struct cgrou
 	struct cgroup_process_info *i;
 	i = lxc_cgroup_process_info_getx("/proc/self/cgroup", meta);
 	if (!i)
-		i = lxc_cgroup_process_info_get(lxc_raw_getpid(), meta);
+		i = lxc_cgroup_process_info_get(getpid(), meta);
 	return i;
 }
 
@@ -1423,12 +1418,11 @@ static bool cgroupfs_mount_cgroup(void *hdata, const char *root, int type)
 	struct cgfs_data *cgfs_d;
 	struct cgroup_process_info *info, *base_info;
 	int r, saved_errno = 0;
-	struct lxc_handler *handler = hdata;
 
 	if (cgns_supported())
 		return true;
 
-	cgfs_d = handler->cgroup_data;
+	cgfs_d = hdata;
 	if (!cgfs_d)
 		return false;
 	base_info = cgfs_d->info;
@@ -2482,7 +2476,7 @@ static bool cgfs_escape(void *hdata)
 		if (!f)
 			goto out;
 
-		written = fprintf(f, "%d\n", lxc_raw_getpid());
+		written = fprintf(f, "%d\n", getpid());
 		fclose(f);
 		if (written < 0) {
 			SYSERROR("writing tasks failed\n");
@@ -2527,14 +2521,14 @@ static bool cgfs_unfreeze(void *hdata)
 	return ret == 0;
 }
 
-static bool cgroupfs_setup_limits(void *hdata, struct lxc_conf *conf,
+static bool cgroupfs_setup_limits(void *hdata, struct lxc_list *cgroup_conf,
 				  bool with_devices)
 {
 	struct cgfs_data *d = hdata;
 
 	if (!d)
 		return false;
-	return do_setup_cgroup_limits(d, &conf->cgroup, with_devices) == 0;
+	return do_setup_cgroup_limits(d, cgroup_conf, with_devices) == 0;
 }
 
 static bool lxc_cgroupfs_attach(const char *name, const char *lxcpath, pid_t pid)
