@@ -21,57 +21,53 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#ifndef _GNU_SOURCE
-#define _GNU_SOURCE 1
-#endif
-#include <caps.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <malloc.h>
-#include <poll.h>
-#include <signal.h>
 #include <stdio.h>
-#include <stdlib.h>
-#include <sys/param.h>
+#include <errno.h>
+#include <unistd.h>
+#include <signal.h>
+#include <fcntl.h>
+#include <poll.h>
 #include <sys/socket.h>
 #include <sys/un.h>
-#include <unistd.h>
+#include <sys/param.h>
+#include <malloc.h>
+#include <stdlib.h>
 
-#include "af_unix.h"
+#include "log.h"
+#include "lxc.h"
+#include "conf.h"
+#include "start.h"	/* for struct lxc_handler */
+#include "utils.h"
 #include "cgroup.h"
 #include "commands.h"
 #include "commands_utils.h"
-#include "conf.h"
-#include "config.h"
-#include "confile.h"
 #include "console.h"
-#include "log.h"
-#include "lxc.h"
+#include "confile.h"
 #include "lxclock.h"
 #include "mainloop.h"
 #include "monitor.h"
-#include "start.h"
-#include "utils.h"
+#include "af_unix.h"
+#include "config.h"
 
 /*
- * This file provides the different functions for clients to query/command the
- * server. The client is typically some lxc tool and the server is typically the
- * container (ie. lxc-start).
+ * This file provides the different functions for clients to
+ * query/command the server. The client is typically some lxc
+ * tool and the server is typically the container (ie. lxc-start).
  *
- * Each command is transactional, the clients send a request to the server and
- * the server answers the request with a message giving the request's status
- * (zero or a negative errno value). Both the request and response may contain
- * additional data.
+ * Each command is transactional, the clients send a request to
+ * the server and the server answers the request with a message
+ * giving the request's status (zero or a negative errno value).
+ * Both the request and response may contain additional data.
  *
- * Each command is wrapped in a ancillary message in order to pass a credential
- * making possible to the server to check if the client is allowed to ask for
- * this command or not.
+ * Each command is wrapped in a ancillary message in order to pass
+ * a credential making possible to the server to check if the client
+ * is allowed to ask for this command or not.
  *
- * IMPORTANTLY: Note that semantics for current commands are fixed. If you wish
- * to make any changes to how, say, LXC_CMD_GET_CONFIG_ITEM works by adding
- * information to the end of cmd.data, then you must introduce a new
- * LXC_CMD_GET_CONFIG_ITEM_V2 define with a new number. You may wish to also
- * mark LXC_CMD_GET_CONFIG_ITEM deprecated in commands.h.
+ * IMPORTANTLY: Note that semantics for current commands are fixed.  If you
+ * wish to make any changes to how, say, LXC_CMD_GET_CONFIG_ITEM works by
+ * adding information to the end of cmd.data, then you must introduce a new
+ * LXC_CMD_GET_CONFIG_ITEM_V2 define with a new number.  You may wish to
+ * also mark LXC_CMD_GET_CONFIG_ITEM deprecated in commands.h.
  *
  * This is necessary in order to avoid having a newly compiled lxc command
  * communicating with a running (old) monitor from crashing the running
@@ -82,25 +78,22 @@ lxc_log_define(lxc_commands, lxc);
 
 static const char *lxc_cmd_str(lxc_cmd_t cmd)
 {
-	static const char *const cmdname[LXC_CMD_MAX] = {
-		[LXC_CMD_CONSOLE]             = "console",
-		[LXC_CMD_CONSOLE_WINCH]       = "console_winch",
-		[LXC_CMD_STOP]                = "stop",
-		[LXC_CMD_GET_STATE]           = "get_state",
-		[LXC_CMD_GET_INIT_PID]        = "get_init_pid",
-		[LXC_CMD_GET_CLONE_FLAGS]     = "get_clone_flags",
-		[LXC_CMD_GET_CGROUP]          = "get_cgroup",
-		[LXC_CMD_GET_CONFIG_ITEM]     = "get_config_item",
-		[LXC_CMD_GET_NAME]            = "get_name",
-		[LXC_CMD_GET_LXCPATH]         = "get_lxcpath",
-		[LXC_CMD_ADD_STATE_CLIENT]    = "add_state_client",
-		[LXC_CMD_CONSOLE_LOG]         = "console_log",
-		[LXC_CMD_SERVE_STATE_CLIENTS] = "serve_state_clients",
+	static const char * const cmdname[LXC_CMD_MAX] = {
+		[LXC_CMD_CONSOLE]          = "console",
+		[LXC_CMD_CONSOLE_WINCH]    = "console_winch",
+		[LXC_CMD_STOP]             = "stop",
+		[LXC_CMD_GET_STATE]        = "get_state",
+		[LXC_CMD_GET_INIT_PID]     = "get_init_pid",
+		[LXC_CMD_GET_CLONE_FLAGS]  = "get_clone_flags",
+		[LXC_CMD_GET_CGROUP]       = "get_cgroup",
+		[LXC_CMD_GET_CONFIG_ITEM]  = "get_config_item",
+		[LXC_CMD_GET_NAME]         = "get_name",
+		[LXC_CMD_GET_LXCPATH]      = "get_lxcpath",
+		[LXC_CMD_ADD_STATE_CLIENT] = "add_state_client",
 	};
 
 	if (cmd >= LXC_CMD_MAX)
-		return "Invalid request";
-
+		return "Unknown cmd";
 	return cmdname[cmd];
 }
 
@@ -128,15 +121,11 @@ static int lxc_cmd_rsp_recv(int sock, struct lxc_cmd_rr *cmd)
 
 	ret = lxc_abstract_unix_recv_fds(sock, &rspfd, 1, rsp, sizeof(*rsp));
 	if (ret < 0) {
-		SYSWARN("Failed to receive response for command \"%s\"",
-		        lxc_cmd_str(cmd->req.cmd));
-
-		if (errno == ECONNRESET)
-			return -1;
-
+		WARN("Command %s failed to receive response: %s.",
+		     lxc_cmd_str(cmd->req.cmd), strerror(errno));
 		return -1;
 	}
-	TRACE("Command \"%s\" received response", lxc_cmd_str(cmd->req.cmd));
+	TRACE("Command \"%s received response", lxc_cmd_str(cmd->req.cmd));
 
 	if (cmd->req.cmd == LXC_CMD_CONSOLE) {
 		struct lxc_cmd_console_rsp_data *rspdata;
@@ -149,42 +138,39 @@ static int lxc_cmd_rsp_recv(int sock, struct lxc_cmd_rr *cmd)
 
 		rspdata = malloc(sizeof(*rspdata));
 		if (!rspdata) {
-			errno = ENOMEM;
-			ERROR("Failed to allocate response buffer for command \"%s\"",
+			ERROR("Command %s couldn't allocate response buffer.",
 			      lxc_cmd_str(cmd->req.cmd));
 			return -1;
 		}
-
 		rspdata->masterfd = rspfd;
 		rspdata->ttynum = PTR_TO_INT(rsp->data);
 		rsp->data = rspdata;
 	}
 
 	if (rsp->datalen == 0) {
-		DEBUG("Response data length for command \"%s\" is 0",
+		DEBUG("command %s response data length is 0",
 		      lxc_cmd_str(cmd->req.cmd));
 		return ret;
 	}
-
 	if (rsp->datalen > LXC_CMD_DATA_MAX) {
-		ERROR("Response data for command \"%s\" is too long: %d bytes > %d",
-		      lxc_cmd_str(cmd->req.cmd), rsp->datalen, LXC_CMD_DATA_MAX);
+		ERROR("Command %s response data %d too long.",
+		      lxc_cmd_str(cmd->req.cmd), rsp->datalen);
+		errno = EFBIG;
 		return -1;
 	}
 
 	rsp->data = malloc(rsp->datalen);
 	if (!rsp->data) {
-		errno = ENOMEM;
-		ERROR("Failed to allocate response buffer for command \"%s\"",
+		ERROR("Command %s was unable to allocate response buffer.",
 		      lxc_cmd_str(cmd->req.cmd));
 		return -1;
 	}
-
 	ret = recv(sock, rsp->data, rsp->datalen, 0);
 	if (ret != rsp->datalen) {
-		SYSERROR("Failed to receive response data for command \"%s\"",
-		         lxc_cmd_str(cmd->req.cmd));
-		return -1;
+		ERROR("Command %s failed to receive response data: %s.",
+		      lxc_cmd_str(cmd->req.cmd), strerror(errno));
+		if (ret >= 0)
+			ret = -1;
 	}
 
 	return ret;
@@ -200,59 +186,73 @@ static int lxc_cmd_rsp_recv(int sock, struct lxc_cmd_rr *cmd)
  */
 static int lxc_cmd_rsp_send(int fd, struct lxc_cmd_rsp *rsp)
 {
-	ssize_t ret;
+	int ret;
 
-	errno = EMSGSIZE;
-	ret = send(fd, rsp, sizeof(*rsp), MSG_NOSIGNAL);
-	if (ret < 0 || (size_t)ret != sizeof(*rsp)) {
-		SYSERROR("Failed to send command response %zd", ret);
+	ret = send(fd, rsp, sizeof(*rsp), 0);
+	if (ret != sizeof(*rsp)) {
+		ERROR("Failed to send command response %d: %s.", ret,
+		      strerror(errno));
 		return -1;
 	}
 
-	if (!rsp->data || rsp->datalen <= 0)
-		return 0;
-
-	ret = send(fd, rsp->data, rsp->datalen, MSG_NOSIGNAL);
-	if (ret < 0 || ret != (ssize_t)rsp->datalen) {
-		SYSWARN("Failed to send command response data %zd", ret);
-		return -1;
+	if (rsp->datalen > 0) {
+		ret = send(fd, rsp->data, rsp->datalen, 0);
+		if (ret != rsp->datalen) {
+			WARN("Failed to send command response data %d: %s.",
+			     ret, strerror(errno));
+			return -1;
+		}
 	}
-
 	return 0;
 }
 
 static int lxc_cmd_send(const char *name, struct lxc_cmd_rr *cmd,
-			const char *lxcpath, const char *hashed_sock_name)
+		 const char *lxcpath, const char *hashed_sock_name)
 {
-	int client_fd, saved_errno;
-	ssize_t ret = -1;
+	int client_fd;
+	int ret = -1;
 
-	client_fd = lxc_cmd_connect(name, lxcpath, hashed_sock_name, "command");
-	if (client_fd < 0)
+	client_fd = lxc_cmd_connect(name, lxcpath, hashed_sock_name);
+	if (client_fd < 0 && client_fd == -ECONNREFUSED)
+		return -ECONNREFUSED;
+	else if (client_fd < 0)
 		return -1;
 
-	ret = lxc_abstract_unix_send_credential(client_fd, &cmd->req,
-						sizeof(cmd->req));
-	if (ret < 0 || (size_t)ret != sizeof(cmd->req))
-		goto on_error;
+	TRACE("Command \"%s\" connected to command socket",
+	      lxc_cmd_str(cmd->req.cmd));
 
-	if (cmd->req.datalen <= 0)
-		return client_fd;
+	ret = lxc_abstract_unix_send_credential(client_fd, &cmd->req, sizeof(cmd->req));
+	if (ret != sizeof(cmd->req)) {
+		close(client_fd);
 
-	errno = EMSGSIZE;
-	ret = send(client_fd, (void *)cmd->req.data, cmd->req.datalen,
-		   MSG_NOSIGNAL);
-	if (ret < 0 || ret != (ssize_t)cmd->req.datalen)
-		goto on_error;
+		if (errno == EPIPE)
+			return -EPIPE;
+
+		if (ret >= 0)
+			return -EMSGSIZE;
+
+		return -1;
+	}
+
+	TRACE("Command \"%s\" requested data of length %d",
+	      lxc_cmd_str(cmd->req.cmd), cmd->req.datalen);
+
+	if (cmd->req.datalen > 0) {
+		ret = send(client_fd, cmd->req.data, cmd->req.datalen, MSG_NOSIGNAL);
+		if (ret != cmd->req.datalen) {
+			close(client_fd);
+
+			if (errno == EPIPE)
+				return -EPIPE;
+
+			if (ret >= 0)
+				return -EMSGSIZE;
+
+			return -1;
+		}
+	}
 
 	return client_fd;
-
-on_error:
-	saved_errno = errno;
-	close(client_fd);
-	errno = saved_errno;
-
-	return -1;
 }
 
 /*
@@ -277,8 +277,7 @@ on_error:
 static int lxc_cmd(const char *name, struct lxc_cmd_rr *cmd, int *stopped,
 		   const char *lxcpath, const char *hashed_sock_name)
 {
-	int client_fd, saved_errno;
-	int ret = -1;
+	int client_fd, ret = -1;
 	bool stay_connected = false;
 
 	if (cmd->req.cmd == LXC_CMD_CONSOLE ||
@@ -287,32 +286,36 @@ static int lxc_cmd(const char *name, struct lxc_cmd_rr *cmd, int *stopped,
 
 	*stopped = 0;
 
+	TRACE("command %s tries to connect command socket",
+	      lxc_cmd_str(cmd->req.cmd));
+
 	client_fd = lxc_cmd_send(name, cmd, lxcpath, hashed_sock_name);
 	if (client_fd < 0) {
-		SYSTRACE("Command \"%s\" failed to connect command socket",
-		         lxc_cmd_str(cmd->req.cmd));
-
-		if (errno == ECONNREFUSED || errno == EPIPE)
+		TRACE("command %s failed to connect command socket: %s",
+		      lxc_cmd_str(cmd->req.cmd), strerror(errno));
+		if (client_fd == -ECONNREFUSED) {
 			*stopped = 1;
+			return -1;
+		}
 
-		return -1;
+		if (client_fd == -EPIPE)
+			goto epipe;
+
+		goto out;
 	}
 
 	ret = lxc_cmd_rsp_recv(client_fd, cmd);
-	if (ret < 0 && errno == ECONNRESET)
-		*stopped = 1;
-
-	if (!stay_connected || ret <= 0) {
-		saved_errno = errno;
+out:
+	if (!stay_connected || ret <= 0)
 		close(client_fd);
-		errno = saved_errno;
-		return ret;
-	}
-
 	if (stay_connected && ret > 0)
 		cmd->rsp.ret = client_fd;
 
 	return ret;
+
+epipe:
+	*stopped = 1;
+	return 0;
 }
 
 int lxc_try_cmd(const char *name, const char *lxcpath)
@@ -323,6 +326,7 @@ int lxc_try_cmd(const char *name, const char *lxcpath)
 	};
 
 	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
+
 	if (stopped)
 		return 0;
 	if (ret > 0 && cmd.rsp.ret < 0) {
@@ -332,16 +336,19 @@ int lxc_try_cmd(const char *name, const char *lxcpath)
 	if (ret > 0)
 		return 0;
 
-	/* At this point we weren't denied access, and the container *was*
-	 * started. There was some inexplicable error in the protocol.  I'm not
-	 * clear on whether we should return -1 here, but we didn't receive a
-	 * -EACCES, so technically it's not that we're not allowed to control
-	 * the container - it's just not behaving.
+	/*
+	 * At this point we weren't denied access, and the
+	 * container *was* started.  There was some inexplicable
+	 * error in the protocol.
+	 * I'm not clear on whether we should return -1 here, but
+	 * we didn't receive a -EACCES, so technically it's not that
+	 * we're not allowed to control the container - it's just not
+	 * behaving.
 	 */
 	return 0;
 }
 
-/* Implementations of the commands and their callbacks */
+/* Implentations of the commands and their callbacks */
 
 /*
  * lxc_cmd_get_init_pid: Get pid of the container's init process
@@ -354,38 +361,21 @@ int lxc_try_cmd(const char *name, const char *lxcpath)
 pid_t lxc_cmd_get_init_pid(const char *name, const char *lxcpath)
 {
 	int ret, stopped;
-	intmax_t pid;
 	struct lxc_cmd_rr cmd = {
-		.req = {
-			.cmd = LXC_CMD_GET_INIT_PID
-		},
-		.rsp = {
-			.data = INTMAX_TO_PTR((intmax_t){-1})
-		}
+		.req = { .cmd = LXC_CMD_GET_INIT_PID },
 	};
 
 	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
 	if (ret < 0)
-		return -1;
+		return ret;
 
-	pid = PTR_TO_INTMAX(cmd.rsp.data);
-	if (pid < 0)
-		return -1;
-
-	/* We need to assume that pid_t can actually hold any pid given to us
-	 * by the kernel. If it can't it's a libc bug.
-	 */
-	return (pid_t)pid;
+	return PTR_TO_INT(cmd.rsp.data);
 }
 
 static int lxc_cmd_get_init_pid_callback(int fd, struct lxc_cmd_req *req,
 					 struct lxc_handler *handler)
 {
-	intmax_t pid = handler->pid;
-
-	struct lxc_cmd_rsp rsp = {
-		.data = INTMAX_TO_PTR(pid)
-	};
+	struct lxc_cmd_rsp rsp = { .data = INT_TO_PTR(handler->pid) };
 
 	return lxc_cmd_rsp_send(fd, &rsp);
 }
@@ -433,31 +423,37 @@ static int lxc_cmd_get_clone_flags_callback(int fd, struct lxc_cmd_req *req,
  * returned path.
  */
 char *lxc_cmd_get_cgroup_path(const char *name, const char *lxcpath,
-			      const char *subsystem)
+	const char *subsystem)
 {
 	int ret, stopped;
 	struct lxc_cmd_rr cmd = {
 		.req = {
 			.cmd = LXC_CMD_GET_CGROUP,
+			.datalen = strlen(subsystem)+1,
 			.data = subsystem,
-			.datalen = 0,
 		},
 	};
 
-	cmd.req.data = subsystem;
-	cmd.req.datalen = 0;
-	if (subsystem)
-		cmd.req.datalen = strlen(subsystem) + 1;
-
 	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
-	if (ret < 0)
+	if (ret < 0) {
+		TRACE("command %s failed for container \"%s\": %s.",
+		      lxc_cmd_str(cmd.req.cmd), name, strerror(errno));
 		return NULL;
+	}
 
-	if (ret == 0)
+	if (!ret) {
+		WARN("container \"%s\" has stopped before sending its state", name);
 		return NULL;
+	}
 
-	if (cmd.rsp.ret < 0 || cmd.rsp.datalen < 0)
+	if (cmd.rsp.ret < 0 || cmd.rsp.datalen < 0) {
+		ERROR("command %s failed for container \"%s\": %s",
+		      lxc_cmd_str(cmd.req.cmd), name, strerror(-cmd.rsp.ret));
 		return NULL;
+	}
+
+	TRACE("command %s successful for container \"%s\"",
+	      lxc_cmd_str(cmd.req.cmd), name);
 
 	return cmd.rsp.data;
 }
@@ -465,19 +461,18 @@ char *lxc_cmd_get_cgroup_path(const char *name, const char *lxcpath,
 static int lxc_cmd_get_cgroup_callback(int fd, struct lxc_cmd_req *req,
 				       struct lxc_handler *handler)
 {
-	const char *path;
 	struct lxc_cmd_rsp rsp;
+	const char *path;
 
-	if (req->datalen > 0)
-		path = cgroup_get_cgroup(handler, req->data);
-	else
-		path = cgroup_get_cgroup(handler, NULL);
-	if (!path)
+	if (req->datalen < 1)
 		return -1;
 
-	rsp.ret = 0;
-	rsp.datalen = strlen(path) + 1;
+	path = cgroup_get_cgroup(handler, req->data);
+	if (!path)
+		return -1;
+	rsp.datalen = strlen(path) + 1,
 	rsp.data = (char *)path;
+	rsp.ret = 0;
 
 	return lxc_cmd_rsp_send(fd, &rsp);
 }
@@ -486,7 +481,7 @@ static int lxc_cmd_get_cgroup_callback(int fd, struct lxc_cmd_req *req,
  * lxc_cmd_get_config_item: Get config item the running container
  *
  * @name     : name of container to connect to
- * @item     : the configuration item to retrieve (ex: lxc.network.0.veth.pair)
+ * @item     : the configuration item to retrieve (ex: lxc.net.0.veth.pair)
  * @lxcpath  : the lxcpath in which the container is running
  *
  * Returns the item on success, NULL on failure. The caller must free() the
@@ -499,7 +494,7 @@ char *lxc_cmd_get_config_item(const char *name, const char *item,
 	struct lxc_cmd_rr cmd = {
 		.req = { .cmd = LXC_CMD_GET_CONFIG_ITEM,
 			 .data = item,
-			 .datalen = strlen(item) + 1,
+			 .datalen = strlen(item)+1,
 		       },
 	};
 
@@ -509,7 +504,6 @@ char *lxc_cmd_get_config_item(const char *name, const char *item,
 
 	if (cmd.rsp.ret == 0)
 		return cmd.rsp.data;
-
 	return NULL;
 }
 
@@ -517,22 +511,21 @@ static int lxc_cmd_get_config_item_callback(int fd, struct lxc_cmd_req *req,
 					    struct lxc_handler *handler)
 {
 	int cilen;
+	struct lxc_cmd_rsp rsp;
 	char *cidata;
 	struct lxc_config_t *item;
-	struct lxc_cmd_rsp rsp;
 
 	memset(&rsp, 0, sizeof(rsp));
-	item = lxc_getconfig(req->data);
+	item = lxc_get_config(req->data);
 	if (!item)
 		goto err1;
-	cilen = item->get(req->data, NULL, 0, handler->conf);
+	cilen = item->get(req->data, NULL, 0, handler->conf, NULL);
 	if (cilen <= 0)
 		goto err1;
 
 	cidata = alloca(cilen + 1);
-	if (item->get(req->data, cidata, cilen + 1, handler->conf) != cilen)
+	if (item->get(req->data, cidata, cilen + 1, handler->conf, NULL) != cilen)
 		goto err1;
-
 	cidata[cilen] = '\0';
 	rsp.data = cidata;
 	rsp.datalen = cilen + 1;
@@ -568,13 +561,12 @@ int lxc_cmd_get_state(const char *name, const char *lxcpath)
 		return -1;
 
 	if (!ret) {
-		WARN("Container \"%s\" has stopped before sending its state", name);
+		WARN("Container \"%s\" has stopped before sending its state.", name);
 		return -1;
 	}
 
-	DEBUG("Container \"%s\" is in \"%s\" state", name,
+	DEBUG("Container \"%s\" is in \"%s\" state.", name,
 	      lxc_state2str(PTR_TO_INT(cmd.rsp.data)));
-
 	return PTR_TO_INT(cmd.rsp.data);
 }
 
@@ -605,23 +597,22 @@ int lxc_cmd_stop(const char *name, const char *lxcpath)
 	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
 	if (ret < 0) {
 		if (stopped) {
-			INFO("Container \"%s\" is already stopped", name);
+			INFO("Container \"%s\" is already stopped.", name);
 			return 0;
 		}
-
 		return -1;
 	}
 
-	/* We do not expect any answer, because we wait for the connection to be
-	 * closed.
+	/* we do not expect any answer, because we wait for the connection to be
+	 * closed
 	 */
 	if (ret > 0) {
-		errno = -cmd.rsp.ret;
-		SYSERROR("Failed to stop container \"%s\"", name);
+		ERROR("Failed to stop container \"%s\": %s.", name,
+		      strerror(-cmd.rsp.ret));
 		return -1;
 	}
 
-	INFO("Container \"%s\" has stopped", name);
+	INFO("Container \"%s\" has stopped.", name);
 	return 0;
 }
 
@@ -636,15 +627,14 @@ static int lxc_cmd_stop_callback(int fd, struct lxc_cmd_req *req,
 	memset(&rsp, 0, sizeof(rsp));
 	rsp.ret = kill(handler->pid, stopsignal);
 	if (!rsp.ret) {
-		/* We can't just use lxc_unfreeze() since we are already in the
+		/* we can't just use lxc_unfreeze() since we are already in the
 		 * context of handling the STOP cmd in lxc-start, and calling
 		 * lxc_unfreeze() would do another cmd (GET_CGROUP) which would
-		 * deadlock us.
+		 * deadlock us
 		 */
 		if (cgroup_unfreeze(handler))
 			return 0;
-
-		ERROR("Failed to unfreeze container \"%s\"", handler->name);
+		ERROR("Failed to unfreeze container \"%s\".", handler->name);
 		rsp.ret = -1;
 	}
 
@@ -679,7 +669,6 @@ static int lxc_cmd_console_winch_callback(int fd, struct lxc_cmd_req *req,
 	struct lxc_cmd_rsp rsp = { .data = 0 };
 
 	lxc_console_sigwinch(SIGWINCH);
-
 	return lxc_cmd_rsp_send(fd, &rsp);
 }
 
@@ -707,29 +696,27 @@ int lxc_cmd_console(const char *name, int *ttynum, int *fd, const char *lxcpath)
 		return ret;
 
 	if (cmd.rsp.ret < 0) {
-		errno = -cmd.rsp.ret;
-		SYSERROR("Denied access to tty");
+		ERROR("Console access denied: %s.", strerror(-cmd.rsp.ret));
 		ret = -1;
 		goto out;
 	}
 
 	if (ret == 0) {
-		ERROR("tty number %d invalid, busy or all ttys busy", *ttynum);
+		ERROR("Console %d invalid, busy or all consoles busy.", *ttynum);
 		ret = -1;
 		goto out;
 	}
 
 	rspdata = cmd.rsp.data;
 	if (rspdata->masterfd < 0) {
-		ERROR("Unable to allocate fd for tty %d", rspdata->ttynum);
+		ERROR("Unable to allocate fd for tty %d.", rspdata->ttynum);
 		goto out;
 	}
 
-	ret = cmd.rsp.ret; /* socket fd */
+	ret = cmd.rsp.ret;	/* sock fd */
 	*fd = rspdata->masterfd;
 	*ttynum = rspdata->ttynum;
-	INFO("Alloced fd %d for tty %d via socket %d", *fd, rspdata->ttynum, ret);
-
+	INFO("tty %d allocated fd %d sock %d.", rspdata->ttynum, *fd, ret);
 out:
 	free(cmd.rsp.data);
 	return ret;
@@ -738,9 +725,9 @@ out:
 static int lxc_cmd_console_callback(int fd, struct lxc_cmd_req *req,
 				    struct lxc_handler *handler)
 {
-	int masterfd, ret;
-	struct lxc_cmd_rsp rsp;
 	int ttynum = PTR_TO_INT(req->data);
+	int masterfd;
+	struct lxc_cmd_rsp rsp;
 
 	masterfd = lxc_console_allocate(handler->conf, fd, &ttynum);
 	if (masterfd < 0)
@@ -748,9 +735,8 @@ static int lxc_cmd_console_callback(int fd, struct lxc_cmd_req *req,
 
 	memset(&rsp, 0, sizeof(rsp));
 	rsp.data = INT_TO_PTR(ttynum);
-	ret = lxc_abstract_unix_send_fds(fd, &masterfd, 1, &rsp, sizeof(rsp));
-	if (ret < 0) {
-		ERROR("Failed to send tty to client");
+	if (lxc_abstract_unix_send_fds(fd, &masterfd, 1, &rsp, sizeof(rsp)) < 0) {
+		ERROR("Failed to send tty to client.");
 		lxc_console_free(handler->conf, fd);
 		goto out_close;
 	}
@@ -758,8 +744,8 @@ static int lxc_cmd_console_callback(int fd, struct lxc_cmd_req *req,
 	return 0;
 
 out_close:
-	/* Special indicator to lxc_cmd_handler() to close the fd and do
-	 * related cleanup.
+	/* special indicator to lxc_cmd_handler() to close the fd and do
+	 * related cleanup
 	 */
 	return 1;
 }
@@ -779,23 +765,23 @@ char *lxc_cmd_get_name(const char *hashed_sock_name)
 	};
 
 	ret = lxc_cmd(NULL, &cmd, &stopped, NULL, hashed_sock_name);
-	if (ret < 0)
+	if (ret < 0) {
 		return NULL;
+	}
 
 	if (cmd.rsp.ret == 0)
 		return cmd.rsp.data;
-
 	return NULL;
 }
 
 static int lxc_cmd_get_name_callback(int fd, struct lxc_cmd_req *req,
-				     struct lxc_handler *handler)
+					    struct lxc_handler *handler)
 {
 	struct lxc_cmd_rsp rsp;
 
 	memset(&rsp, 0, sizeof(rsp));
 
-	rsp.data = (char *)handler->name;
+	rsp.data = handler->name;
 	rsp.datalen = strlen(handler->name) + 1;
 	rsp.ret = 0;
 
@@ -817,12 +803,12 @@ char *lxc_cmd_get_lxcpath(const char *hashed_sock_name)
 	};
 
 	ret = lxc_cmd(NULL, &cmd, &stopped, NULL, hashed_sock_name);
-	if (ret < 0)
+	if (ret < 0) {
 		return NULL;
+	}
 
 	if (cmd.rsp.ret == 0)
 		return cmd.rsp.data;
-
 	return NULL;
 }
 
@@ -833,9 +819,9 @@ static int lxc_cmd_get_lxcpath_callback(int fd, struct lxc_cmd_req *req,
 
 	memset(&rsp, 0, sizeof(rsp));
 
-	rsp.ret = 0;
 	rsp.data = (char *)handler->lxcpath;
 	rsp.datalen = strlen(handler->lxcpath) + 1;
+	rsp.ret = 0;
 
 	return lxc_cmd_rsp_send(fd, &rsp);
 }
@@ -844,8 +830,9 @@ int lxc_cmd_add_state_client(const char *name, const char *lxcpath,
 			     lxc_state_t states[MAX_STATE],
 			     int *state_client_fd)
 {
-	int state, stopped;
+	int stopped;
 	ssize_t ret;
+	int state = -1;
 	struct lxc_cmd_rr cmd = {
 	    .req = {
 		.cmd     = LXC_CMD_ADD_STATE_CLIENT,
@@ -854,126 +841,95 @@ int lxc_cmd_add_state_client(const char *name, const char *lxcpath,
 	    },
 	};
 
-	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
-	if (states[STOPPED] != 0 && stopped != 0)
-		return STOPPED;
-
-	if (ret < 0) {
-		if (errno != ECONNREFUSED)
-			SYSERROR("Failed to execute command");
-
-		return -1;
-	}
-
-	/* We should now be guaranteed to get an answer from the state sending
-	 * function.
+	/* Lock the whole lxc_cmd_add_state_client_callback() call to ensure
+	*  that lxc_set_state() doesn't cause us to miss a state.
 	 */
-	if (cmd.rsp.ret < 0) {
-		errno = -cmd.rsp.ret;
-		SYSERROR("Failed to receive socket fd");
+	process_lock();
+	/* Check if already in requested state. */
+	state = lxc_getstate(name, lxcpath);
+	if (state < 0) {
+		process_unlock();
+		TRACE("failed to retrieve state of container: %s",
+		      strerror(errno));
 		return -1;
-	}
-
-	state = PTR_TO_INT(cmd.rsp.data);
-	if (state < MAX_STATE) {
-		TRACE("Container is already in requested state %s", lxc_state2str(state));
-		close(cmd.rsp.ret);
+	} else if (states[state]) {
+		process_unlock();
+		TRACE("container is %s state", lxc_state2str(state));
 		return state;
 	}
 
+	if ((state == STARTING) && !states[RUNNING] && !states[STOPPING] && !states[STOPPED]) {
+		process_unlock();
+		TRACE("container is in %s state and caller requested to be "
+		      "informed about a previous state",
+		      lxc_state2str(state));
+		return state;
+	} else if ((state == RUNNING) && !states[STOPPING] && !states[STOPPED]) {
+		process_unlock();
+		TRACE("container is in %s state and caller requested to be "
+		      "informed about a previous state",
+		      lxc_state2str(state));
+		return state;
+	} else if ((state == STOPPING) && !states[STOPPED]) {
+		process_unlock();
+		TRACE("container is in %s state and caller requested to be "
+		      "informed about a previous state",
+		      lxc_state2str(state));
+		return state;
+	} else if ((state == STOPPED) || (state == ABORTING)) {
+		process_unlock();
+		TRACE("container is in %s state and caller requested to be "
+		      "informed about a previous state",
+		      lxc_state2str(state));
+		return state;
+	}
+
+	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
+	process_unlock();
+	if (ret < 0) {
+		ERROR("failed to execute command: %s", strerror(errno));
+		return -1;
+	}
+	/* We should now be guaranteed to get an answer from the state sending
+	 * function.
+	 */
+
+	if (cmd.rsp.ret < 0) {
+		ERROR("failed to receive socket fd");
+		return -1;
+	}
+
 	*state_client_fd = cmd.rsp.ret;
-	TRACE("Added state client %d to state client list", cmd.rsp.ret);
 	return MAX_STATE;
 }
 
 static int lxc_cmd_add_state_client_callback(int fd, struct lxc_cmd_req *req,
 					     struct lxc_handler *handler)
 {
-	int ret;
 	struct lxc_cmd_rsp rsp = {0};
 
-	if (req->datalen < 0)
-		goto reap_client_fd;
-
-	if (req->datalen > (sizeof(lxc_state_t) * MAX_STATE))
-		goto reap_client_fd;
-
-	if (!req->data)
-		goto reap_client_fd;
-
-	rsp.ret = lxc_add_state_client(fd, handler, (lxc_state_t *)req->data);
-	if (rsp.ret < 0)
-		goto reap_client_fd;
-
-	rsp.data = INT_TO_PTR(rsp.ret);
-
-	ret = lxc_cmd_rsp_send(fd, &rsp);
-	if (ret < 0)
-		goto reap_client_fd;
-
-	return 0;
-
-reap_client_fd:
-	/* Special indicator to lxc_cmd_handler() to close the fd and do related
-	 * cleanup.
-	 */
-	return 1;
-}
-
-static int lxc_cmd_console_log_callback(int fd, struct lxc_cmd_req *req,
-					struct lxc_handler *handler)
-{
-	struct lxc_cmd_rsp rsp = {0};
-
-	rsp.ret = -ENOSYS;
-	lxc_cmd_rsp_send(fd, &rsp);
-
-	return -ENOSYS;
-}
-
-int lxc_cmd_serve_state_clients(const char *name, const char *lxcpath,
-				lxc_state_t state)
-{
-	int stopped;
-	ssize_t ret;
-	struct lxc_cmd_rr cmd = {
-	    .req = {
-		.cmd  = LXC_CMD_SERVE_STATE_CLIENTS,
-		.data = INT_TO_PTR(state)
-	    },
-	};
-
-	ret = lxc_cmd(name, &cmd, &stopped, lxcpath, NULL);
-	if (ret < 0) {
-		SYSERROR("Failed to execute command");
+	if (req->datalen < 0) {
+		TRACE("Requested datalen was < 0");
 		return -1;
 	}
 
-	return 0;
-}
+	if (req->datalen > (sizeof(lxc_state_t) * MAX_STATE)) {
+		TRACE("Requested datalen was too large");
+		return -1;
+	}
 
-static int lxc_cmd_serve_state_clients_callback(int fd, struct lxc_cmd_req *req,
-						struct lxc_handler *handler)
-{
-	int ret;
-	lxc_state_t state = PTR_TO_INT(req->data);
-	struct lxc_cmd_rsp rsp = {0};
+	if (!req->data) {
+		TRACE("No states requested");
+		return -1;
+	}
 
-	ret = lxc_serve_state_clients(handler->name, handler, state);
-	if (ret < 0)
-		goto reap_client_fd;
+	rsp.ret = lxc_add_state_client(fd, handler, (lxc_state_t *)req->data);
+	if (rsp.ret < 0)
+		ERROR("Failed to add state client %d to state client list", fd);
+	else
+		TRACE("Added state client %d to state client list", fd);
 
-	ret = lxc_cmd_rsp_send(fd, &rsp);
-	if (ret < 0)
-		goto reap_client_fd;
-
-	return 0;
-
-reap_client_fd:
-	/* Special indicator to lxc_cmd_handler() to close the fd and do related
-	 * cleanup.
-	 */
-	return 1;
+	return lxc_cmd_rsp_send(fd, &rsp);
 }
 
 static int lxc_cmd_process(int fd, struct lxc_cmd_req *req,
@@ -982,57 +938,32 @@ static int lxc_cmd_process(int fd, struct lxc_cmd_req *req,
 	typedef int (*callback)(int, struct lxc_cmd_req *, struct lxc_handler *);
 
 	callback cb[LXC_CMD_MAX] = {
-		[LXC_CMD_CONSOLE]             = lxc_cmd_console_callback,
-		[LXC_CMD_CONSOLE_WINCH]       = lxc_cmd_console_winch_callback,
-		[LXC_CMD_STOP]                = lxc_cmd_stop_callback,
-		[LXC_CMD_GET_STATE]           = lxc_cmd_get_state_callback,
-		[LXC_CMD_GET_INIT_PID]        = lxc_cmd_get_init_pid_callback,
-		[LXC_CMD_GET_CLONE_FLAGS]     = lxc_cmd_get_clone_flags_callback,
-		[LXC_CMD_GET_CGROUP]          = lxc_cmd_get_cgroup_callback,
-		[LXC_CMD_GET_CONFIG_ITEM]     = lxc_cmd_get_config_item_callback,
-		[LXC_CMD_GET_NAME]            = lxc_cmd_get_name_callback,
-		[LXC_CMD_GET_LXCPATH]         = lxc_cmd_get_lxcpath_callback,
-		[LXC_CMD_ADD_STATE_CLIENT]    = lxc_cmd_add_state_client_callback,
-		[LXC_CMD_CONSOLE_LOG]         = lxc_cmd_console_log_callback,
-		[LXC_CMD_SERVE_STATE_CLIENTS] = lxc_cmd_serve_state_clients_callback,
+		[LXC_CMD_CONSOLE]          = lxc_cmd_console_callback,
+		[LXC_CMD_CONSOLE_WINCH]    = lxc_cmd_console_winch_callback,
+		[LXC_CMD_STOP]             = lxc_cmd_stop_callback,
+		[LXC_CMD_GET_STATE]        = lxc_cmd_get_state_callback,
+		[LXC_CMD_GET_INIT_PID]     = lxc_cmd_get_init_pid_callback,
+		[LXC_CMD_GET_CLONE_FLAGS]  = lxc_cmd_get_clone_flags_callback,
+		[LXC_CMD_GET_CGROUP]       = lxc_cmd_get_cgroup_callback,
+		[LXC_CMD_GET_CONFIG_ITEM]  = lxc_cmd_get_config_item_callback,
+		[LXC_CMD_GET_NAME]         = lxc_cmd_get_name_callback,
+		[LXC_CMD_GET_LXCPATH]      = lxc_cmd_get_lxcpath_callback,
+		[LXC_CMD_ADD_STATE_CLIENT] = lxc_cmd_add_state_client_callback,
 	};
 
 	if (req->cmd >= LXC_CMD_MAX) {
-		ERROR("Undefined command id %d", req->cmd);
+		ERROR("Undefined command id %d received.", req->cmd);
 		return -1;
 	}
 	return cb[req->cmd](fd, req, handler);
 }
 
 static void lxc_cmd_fd_cleanup(int fd, struct lxc_handler *handler,
-			       struct lxc_epoll_descr *descr,
-			       const lxc_cmd_t cmd)
+			       struct lxc_epoll_descr *descr)
 {
-	struct state_client *client;
-	struct lxc_list *cur, *next;
-
 	lxc_console_free(handler->conf, fd);
 	lxc_mainloop_del_handler(descr, fd);
-	if (cmd != LXC_CMD_ADD_STATE_CLIENT) {
-		close(fd);
-		return;
-	}
-
-	lxc_list_for_each_safe(cur, &handler->state_clients, next) {
-		client = cur->elem;
-		if (client->clientfd != fd)
-			continue;
-
-		/* kick client from list */
-		lxc_list_del(cur);
-		close(client->clientfd);
-		free(cur->elem);
-		free(cur);
-		/* No need to walk the whole list. If we found the state client
-		 * fd there can't be a second one.
-		 */
-		break;
-	}
+	close(fd);
 }
 
 static int lxc_cmd_handler(int fd, uint32_t events, void *data,
@@ -1043,35 +974,39 @@ static int lxc_cmd_handler(int fd, uint32_t events, void *data,
 	struct lxc_handler *handler = data;
 
 	ret = lxc_abstract_unix_rcv_credential(fd, &req, sizeof(req));
-	if (ret < 0) {
-		SYSERROR("Failed to receive data on command socket for command "
-		         "\"%s\"", lxc_cmd_str(req.cmd));
-
-		if (errno == EACCES) {
-			/* We don't care for the peer, just send and close. */
-			struct lxc_cmd_rsp rsp = {.ret = ret};
-
-			lxc_cmd_rsp_send(fd, &rsp);
-		}
+	if (ret == -EACCES) {
+		/* we don't care for the peer, just send and close */
+		struct lxc_cmd_rsp rsp = { .ret = ret };
 
+		lxc_cmd_rsp_send(fd, &rsp);
 		goto out_close;
 	}
 
-	if (ret == 0)
+	TRACE("Processing \"%s\" command", lxc_cmd_str(req.cmd));
+	if (ret < 0) {
+		SYSERROR("Failed to receive data on command socket for \"%s\"",
+			 lxc_cmd_str(req.cmd));
 		goto out_close;
+	}
+
+	if (!ret) {
+		DEBUG("Peer has disconnected for \"%s\"", lxc_cmd_str(req.cmd));
+		goto out_close;
+	}
 
 	if (ret != sizeof(req)) {
 		WARN("Failed to receive full command request. Ignoring request "
-		     "for \"%s\"", lxc_cmd_str(req.cmd));
+		     "for \"%s\"",
+		     lxc_cmd_str(req.cmd));
 		ret = -1;
 		goto out_close;
 	}
 
 	if (req.datalen > LXC_CMD_DATA_MAX) {
 		ERROR("Received command data length %d is too large for "
-		      "command \"%s\"", req.datalen, lxc_cmd_str(req.cmd));
-		errno = EFBIG;
-		ret = -EFBIG;
+		      "command \"%s\"",
+		      req.datalen, lxc_cmd_str(req.cmd));
+		ret = -1;
 		goto out_close;
 	}
 
@@ -1082,34 +1017,32 @@ static int lxc_cmd_handler(int fd, uint32_t events, void *data,
 		ret = recv(fd, reqdata, req.datalen, 0);
 		if (ret != req.datalen) {
 			WARN("Failed to receive full command request. Ignoring "
-			     "request for \"%s\"", lxc_cmd_str(req.cmd));
+			     "request for \"%s\"",
+			     lxc_cmd_str(req.cmd));
 			ret = -1;
 			goto out_close;
 		}
-
 		req.data = reqdata;
 	}
 
 	ret = lxc_cmd_process(fd, &req, handler);
 	if (ret) {
-		/* This is not an error, but only a request to close fd. */
+		/* this is not an error, but only a request to close fd */
 		ret = 0;
 		goto out_close;
 	}
 
 out:
 	return ret;
-
 out_close:
-	lxc_cmd_fd_cleanup(fd, handler, descr, req.cmd);
+	lxc_cmd_fd_cleanup(fd, handler, descr);
 	goto out;
 }
 
 static int lxc_cmd_accept(int fd, uint32_t events, void *data,
 			  struct lxc_epoll_descr *descr)
 {
-	int connection;
-	int opt = 1, ret = -1;
+	int opt = 1, ret = -1, connection;
 
 	connection = accept(fd, NULL, 0);
 	if (connection < 0) {
@@ -1117,21 +1050,20 @@ static int lxc_cmd_accept(int fd, uint32_t events, void *data,
 		return -1;
 	}
 
-	ret = fcntl(connection, F_SETFD, FD_CLOEXEC);
-	if (ret < 0) {
-		SYSERROR("Failed to set close-on-exec on incoming command connection");
+	if (fcntl(connection, F_SETFD, FD_CLOEXEC)) {
+		SYSERROR("Failed to set close-on-exec on incoming command connection.");
 		goto out_close;
 	}
 
-	ret = setsockopt(connection, SOL_SOCKET, SO_PASSCRED, &opt, sizeof(opt));
-	if (ret < 0) {
-		SYSERROR("Failed to enable necessary credentials on command socket");
+	if (setsockopt(connection, SOL_SOCKET,
+		       SO_PASSCRED, &opt, sizeof(opt))) {
+		SYSERROR("Failed to enable necessary credentials on command socket.");
 		goto out_close;
 	}
 
 	ret = lxc_mainloop_add_handler(descr, connection, lxc_cmd_handler, data);
 	if (ret) {
-		ERROR("Failed to add command handler");
+		ERROR("Failed to add command handler.");
 		goto out_close;
 	}
 
@@ -1143,44 +1075,52 @@ out_close:
 	goto out;
 }
 
-int lxc_cmd_init(const char *name, const char *lxcpath, const char *suffix)
+int lxc_cmd_init(const char *name, struct lxc_handler *handler,
+		 const char *lxcpath)
 {
-	int fd, ret;
-	char path[LXC_AUDS_ADDR_LEN] = {0};
+	int fd;
+	char path[sizeof(((struct sockaddr_un *)0)->sun_path)] = { 0 };
+	char *offset = &path[1];
+	int len;
 
-	ret = lxc_make_abstract_socket_name(path, sizeof(path), name, lxcpath, NULL, suffix);
-	if (ret < 0)
+	/* -2 here because this is an abstract unix socket so it needs a
+	 * leading \0, and we null terminate, so it needs a trailing \0.
+	 * Although null termination isn't required by the API, we do it anyway
+	 * because we print the sockname out sometimes.
+	 */
+	len = sizeof(path) - 2;
+	if (lxc_make_abstract_socket_name(offset, len, name, lxcpath, NULL,
+					  "command"))
 		return -1;
-	TRACE("Creating abstract unix socket \"%s\"", &path[1]);
 
 	fd = lxc_abstract_unix_open(path, SOCK_STREAM, 0);
 	if (fd < 0) {
-		SYSERROR("Failed to create command socket %s", &path[1]);
+		ERROR("Failed to create the command service point %s: %s.",
+		      offset, strerror(errno));
 		if (errno == EADDRINUSE)
-			ERROR("Container \"%s\" appears to be already running", name);
-
+			ERROR("Container \"%s\" appears to be already running!", name);
 		return -1;
 	}
 
-	ret = fcntl(fd, F_SETFD, FD_CLOEXEC);
-	if (ret < 0) {
-		SYSERROR("Failed to set FD_CLOEXEC on command socket file descriptor");
+	if (fcntl(fd, F_SETFD, FD_CLOEXEC)) {
+		SYSERROR("Failed to set FD_CLOEXEC on signal file descriptor.");
 		close(fd);
 		return -1;
 	}
 
-	return fd;
+	handler->conf->maincmd_fd = fd;
+	return 0;
 }
 
-int lxc_cmd_mainloop_add(const char *name, struct lxc_epoll_descr *descr,
+int lxc_cmd_mainloop_add(const char *name,
+			 struct lxc_epoll_descr *descr,
 			 struct lxc_handler *handler)
 {
-	int ret;
-	int fd = handler->conf->maincmd_fd;
+	int ret, fd = handler->conf->maincmd_fd;
 
 	ret = lxc_mainloop_add_handler(descr, fd, lxc_cmd_accept, handler);
-	if (ret < 0) {
-		ERROR("Failed to add handler for command socket");
+	if (ret) {
+		ERROR("Failed to add handler for command socket.");
 		close(fd);
 	}
 
