@@ -24,9 +24,6 @@
 #ifndef __LXC_CONSOLE_H
 #define __LXC_CONSOLE_H
 
-#include <signal.h>
-#include <stdio.h>
-
 #include "conf.h"
 #include "list.h"
 
@@ -41,21 +38,18 @@ struct lxc_tty_state
 	/* Escape sequence to use for exiting the pty. A single char can be
 	 * specified. The pty can then exited by doing: Ctrl + specified_char + q.
 	 * This field is checked by lxc_console_cb_tty_stdin(). Set to -1 to
-	 * disable exiting the pty via a escape sequence.
-	 */
+	 * disable exiting the pty via a escape sequence. */
 	int escape;
 	/* Used internally by lxc_console_cb_tty_stdin() to check whether an
-	 * escape sequence has been received.
-	 */
+	 * escape sequence has been received. */
 	int saw_escape;
 	/* Name of the container to forward the SIGWINCH event to. */
 	const char *winch_proxy;
 	/* Path of the container to forward the SIGWINCH event to. */
 	const char *winch_proxy_lxcpath;
-	/* File descriptor that accepts signals. If set to -1 no signal handler
-	 * could be installed. This also means that the sigset_t oldmask member
-	 * is meaningless.
-	 */
+	/* File descriptor that accepts SIGWINCH signals. If set to -1 no
+	 * SIGWINCH handler could be installed. This also means that
+	 * the sigset_t oldmask member is meaningless. */
 	int sigfd;
 	sigset_t oldmask;
 };
@@ -82,12 +76,6 @@ extern int  lxc_console_allocate(struct lxc_conf *conf, int sockfd, int *ttynum)
  * automatically chowned to the uid/gid of the unprivileged user. For this
  * ttys_shift_ids() can be called.)
  */
-extern int lxc_pty_create(struct lxc_console *console);
-
-/**
- * lxc_console_create: Create a new pty.
- * - In addition to lxc_pty_create() also sets up all pty logs.
- */
 extern int  lxc_console_create(struct lxc_conf *);
 
 /*
@@ -114,7 +102,7 @@ extern void lxc_console_free(struct lxc_conf *conf, int fd);
 /*
  * Register pty event handlers in an open mainloop
  */
-extern int  lxc_console_mainloop_add(struct lxc_epoll_descr *, struct lxc_console *);
+extern int  lxc_console_mainloop_add(struct lxc_epoll_descr *, struct lxc_conf *);
 
 /*
  * Handle SIGWINCH events on the allocated ptys.
@@ -122,7 +110,7 @@ extern int  lxc_console_mainloop_add(struct lxc_epoll_descr *, struct lxc_consol
 extern void lxc_console_sigwinch(int sig);
 
 /*
- * Connect to one of the ptys given to the container via lxc.tty.
+ * Connect to one of the ptys given to the container via lxc.tty.max.
  * - allocates either the current controlling pty (default) or a user specified
  *   pty as peer pty for the containers tty
  * - sets up SIGWINCH handler, winsz, and new terminal settings
@@ -135,10 +123,10 @@ extern int  lxc_console(struct lxc_container *c, int ttynum,
 		        int escape);
 
 /*
- * Allocate one of the ptys given to the container via lxc.tty. Returns an open
- * fd to the allocated pty.
+ * Allocate one of the ptys given to the container via lxc.tty.max. Returns an
+ * open fd to the allocated pty.
  * Set ttynum to -1 to allocate the first available pty, or to a value within
- * the range specified by lxc.tty to allocate a specific pty.
+ * the range specified by lxc.tty.max to allocate a specific pty.
  */
 extern int  lxc_console_getfd(struct lxc_container *c, int *ttynum,
 			      int *masterfd);
@@ -184,57 +172,47 @@ extern int lxc_setup_tios(int fd, struct termios *oldtios);
 extern void lxc_console_winsz(int srcfd, int dstfd);
 
 /*
- * lxc_console_signal_init: install signal handler
+ * lxc_console_sigwinch_init: install SIGWINCH handler
  *
  * @srcfd  : src for winsz in SIGWINCH handler
  * @dstfd  : dst for winsz in SIGWINCH handler
  *
  * Returns lxc_tty_state structure on success or NULL on failure. The sigfd
  * member of the returned lxc_tty_state can be select()/poll()ed/epoll()ed
- * on (ie added to a mainloop) for signals.
+ * on (ie added to a mainloop) for SIGWINCH.
  *
  * Must be called with process_lock held to protect the lxc_ttys list, or
  * from a non-threaded context.
  *
- * Note that the signal handler isn't installed as a classic asychronous
- * handler, rather signalfd(2) is used so that we can handle the signal when
- * we're ready for it. This avoids deadlocks since a signal handler (ie
- * lxc_console_sigwinch()) would need to take the thread mutex to prevent
- * lxc_ttys list corruption, but using the fd we can provide the tty_state
- * needed to the callback (lxc_console_cb_signal_fd()).
+ * Note that SIGWINCH isn't installed as a classic asychronous handler,
+ * rather signalfd(2) is used so that we can handle the signal when we're
+ * ready for it. This avoids deadlocks since a signal handler
+ * (ie lxc_console_sigwinch()) would need to take the thread mutex to
+ * prevent lxc_ttys list corruption, but using the fd we can provide the
+ * tty_state needed to the callback (lxc_console_cb_sigwinch_fd()).
  *
  * This function allocates memory. It is up to the caller to free it.
  */
-extern struct lxc_tty_state *lxc_console_signal_init(int srcfd, int dstfd);
+extern struct lxc_tty_state *lxc_console_sigwinch_init(int srcfd, int dstfd);
 
 /*
- * Handler for signal events. To be registered via the corresponding functions
+ * Handler for SIGWINCH events. To be registered via the corresponding functions
  * declared and defined in mainloop.{c,h} or lxc_console_mainloop_add().
  */
-extern int lxc_console_cb_signal_fd(int fd, uint32_t events, void *cbdata,
-				    struct lxc_epoll_descr *descr);
+extern int lxc_console_cb_sigwinch_fd(int fd, uint32_t events, void *cbdata,
+		struct lxc_epoll_descr *descr);
 
 /*
- * lxc_console_signal_fini: uninstall signal handler
+ * lxc_console_sigwinch_fini: uninstall SIGWINCH handler
  *
- * @ts  : the lxc_tty_state returned by lxc_console_signal_init
+ * @ts  : the lxc_tty_state returned by lxc_console_sigwinch_init
  *
  * Restore the saved signal handler that was in effect at the time
- * lxc_console_signal_init() was called.
+ * lxc_console_sigwinch_init() was called.
  *
  * Must be called with process_lock held to protect the lxc_ttys list, or
  * from a non-threaded context.
  */
-extern void lxc_console_signal_fini(struct lxc_tty_state *ts);
-
-extern int lxc_console_cb_con(int fd, uint32_t events, void *data,
-			      struct lxc_epoll_descr *descr);
-
-extern int lxc_make_controlling_pty(int fd);
-extern int lxc_login_pty(int fd);
-extern void lxc_pty_conf_free(struct lxc_console *console);
-extern void lxc_pty_info_init(struct lxc_pty_info *pty);
-extern void lxc_pty_init(struct lxc_console *pty);
-extern int lxc_pty_map_ids(struct lxc_conf *c, struct lxc_console *pty);
+extern void lxc_console_sigwinch_fini(struct lxc_tty_state *ts);
 
 #endif
