@@ -21,7 +21,6 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#define _GNU_SOURCE
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -110,37 +109,19 @@ static int fillwaitedstates(const char *strstates, lxc_state_t *states)
 extern int lxc_wait(const char *lxcname, const char *states, int timeout,
 		    const char *lxcpath)
 {
-	int state = -1;
+	int state;
 	lxc_state_t s[MAX_STATE] = {0};
 
 	if (fillwaitedstates(states, s))
 		return -1;
 
-	for (;;) {
-		state = lxc_cmd_sock_get_state(lxcname, lxcpath, s, timeout);
-		if (state >= 0)
-			break;
-
-		if (errno != ECONNREFUSED) {
-			SYSERROR("Failed to receive state from monitor");
-			return -1;
-		}
-
-		if (timeout > 0)
-			timeout--;
-
-		if (timeout == 0)
-			return -1;
-
-		sleep(1);
-	}
-
+	state = lxc_cmd_sock_get_state(lxcname, lxcpath, s, timeout);
 	if (state < 0) {
-		ERROR("Failed to retrieve state from monitor");
+		SYSERROR("failed to receive state from monitor");
 		return -1;
 	}
 
-	TRACE("Retrieved state of container %s", lxc_state2str(state));
+	TRACE("retrieved state of container %s", lxc_state2str(state));
 	if (!s[state])
 		return -1;
 
