@@ -39,102 +39,97 @@
 
 lxc_log_define(rsync, lxc);
 
-/* the bulk of this needs to become a common helper */
-int do_rsync(const char *src, const char *dest)
+int lxc_storage_rsync_exec_wrapper(void *data)
 {
-	// call out to rsync
-	pid_t pid;
-	char *s;
-	size_t l;
+	struct rsync_data *arg = data;
+	return lxc_rsync(arg);
+}
 
-	pid = fork();
-	if (pid < 0)
+int lxc_rsync_exec_wrapper(void *data)
+{
+	int ret;
+	struct rsync_data_char *args = data;
+
+	ret = lxc_switch_uid_gid(0, 0);
+	if (ret < 0)
 		return -1;
-	if (pid > 0)
-		return wait_for_pid(pid);
+
+	ret = lxc_setgroups(0, NULL);
+	if (ret < 0)
+		return -1;
+
+	return lxc_rsync_exec(args->src, args->dest);
+}
+
+int lxc_rsync_exec(const char *src, const char *dest)
+{
+	int ret;
+	size_t l;
+	char *s;
 
 	l = strlen(src) + 2;
 	s = malloc(l);
 	if (!s)
-		exit(1);
-	strcpy(s, src);
-	s[l-2] = '/';
-	s[l-1] = '\0';
+		return -1;
+
+	ret = snprintf(s, l, "%s", src);
+	if (ret < 0 || (size_t)ret >= l)
+		return -1;
+
+	s[l - 2] = '/';
+	s[l - 1] = '\0';
 
 	execlp("rsync", "rsync", "-aHXS", "--delete", s, dest, (char *)NULL);
-	exit(1);
+	return -1;
 }
 
-int rsync_delta(struct rsync_data_char *data)
-{
-	if (setgid(0) < 0) {
-		ERROR("Failed to setgid to 0");
-		return -1;
-	}
-	if (setgroups(0, NULL) < 0)
-		WARN("Failed to clear groups");
-	if (setuid(0) < 0) {
-		ERROR("Failed to setuid to 0");
-		return -1;
-	}
-	if (do_rsync(data->src, data->dest) < 0) {
-		ERROR("rsyncing %s to %s", data->src, data->dest);
-		return -1;
-	}
-
-	return 0;
-}
-
-int rsync_delta_wrapper(void *data)
-{
-	struct rsync_data_char *arg = data;
-	return rsync_delta(arg);
-}
-
-int rsync_rootfs(struct rsync_data *data)
+int lxc_rsync(struct rsync_data *data)
 {
+	int ret;
+	char *dest, *src;
 	struct lxc_storage *orig = data->orig, *new = data->new;
 
-	if (unshare(CLONE_NEWNS) < 0) {
-		SYSERROR("unshare CLONE_NEWNS");
+	ret = unshare(CLONE_NEWNS);
+	if (ret < 0) {
+		SYSERROR("Failed to unshare CLONE_NEWNS");
 		return -1;
 	}
-	if (detect_shared_rootfs()) {
-		if (mount(NULL, "/", NULL, MS_SLAVE|MS_REC, NULL)) {
-			SYSERROR("Failed to make / rslave");
-			ERROR("Continuing...");
-		}
-	}
 
-	// If not a snapshot, copy the fs.
-	if (orig->ops->mount(orig) < 0) {
-		ERROR("failed mounting %s onto %s", orig->src, orig->dest);
+	ret = detect_shared_rootfs();
+	if (ret) {
+		ret = mount(NULL, "/", NULL, MS_SLAVE|MS_REC, NULL);
+		if (ret < 0)
+			SYSERROR("Failed to make \"/\" a slave mount");
+	}
+
+	ret = orig->ops->mount(orig);
+	if (ret < 0) {
+		ERROR("Failed mounting \"%s\" on \"%s\"", orig->src, orig->dest);
 		return -1;
 	}
-	if (new->ops->mount(new) < 0) {
-		ERROR("failed mounting %s onto %s", new->src, new->dest);
+
+	ret = new->ops->mount(new);
+	if (ret < 0) {
+		ERROR("Failed mounting \"%s\" onto \"%s\"", new->src, new->dest);
 		return -1;
 	}
-	if (setgid(0) < 0) {
-		ERROR("Failed to setgid to 0");
+
+	ret = lxc_switch_uid_gid(0, 0);
+	if (ret < 0)
 		return -1;
-	}
-	if (setgroups(0, NULL) < 0)
-		WARN("Failed to clear groups");
-	if (setuid(0) < 0) {
-		ERROR("Failed to setuid to 0");
+
+	ret = lxc_setgroups(0, NULL);
+	if (ret < 0)
 		return -1;
-	}
-	if (do_rsync(orig->dest, new->dest) < 0) {
-		ERROR("rsyncing %s to %s", orig->src, new->src);
+
+	src = lxc_storage_get_path(orig->dest, orig->type);
+	dest = lxc_storage_get_path(new->dest, new->type);
+
+	ret = lxc_rsync_exec(src, dest);
+	if (ret < 0) {
+		ERROR("Failed to rsync from \"%s\" into \"%s\"", src, dest);
 		return -1;
 	}
 
 	return 0;
 }
-
-int rsync_rootfs_wrapper(void *data)
-{
-	struct rsync_data *arg = data;
-	return rsync_rootfs(arg);
-}
