@@ -46,10 +46,6 @@
 #include "storage_utils.h"
 #include "utils.h"
 
-#ifndef HAVE_STRLCPY
-#include "include/strlcpy.h"
-#endif
-
 #ifndef BLKGETSIZE64
 #define BLKGETSIZE64 _IOR(0x12, 114, size_t)
 #endif
@@ -91,7 +87,6 @@ char *dir_new_path(char *src, const char *oldname, const char *name,
 	while ((p2 = strstr(src, oldname)) != NULL) {
 		/* copy text up to oldname */
 		strncpy(p, src, p2 - src);
-
 		/* move target pointer (p) */
 		p += p2 - src;
 		/* print new name in place of oldname */
@@ -99,7 +94,6 @@ char *dir_new_path(char *src, const char *oldname, const char *name,
 		/* move src to end of oldname */
 		src = p2 + l2;
 	}
-
 	/* copy the rest of src */
 	sprintf(p, "%s", src);
 	return ret;
@@ -139,16 +133,15 @@ bool attach_block_device(struct lxc_conf *conf)
 int blk_getsize(struct lxc_storage *bdev, uint64_t *size)
 {
 	int fd, ret;
-	char *path = bdev->src;
+	char *src;
 
-	if (strcmp(bdev->type, "loop") == 0)
-		path = bdev->src + 5;
-
-	fd = open(path, O_RDONLY);
+	src = lxc_storage_get_path(bdev->src, bdev->type);
+	fd = open(src, O_RDONLY);
 	if (fd < 0)
 		return -1;
 
-	ret = ioctl(fd, BLKGETSIZE64, size); // size of device in bytes
+	/* size of device in bytes */
+	ret = ioctl(fd, BLKGETSIZE64, size);
 	close(fd);
 	return ret;
 }
@@ -173,15 +166,12 @@ int detect_fs(struct lxc_storage *bdev, char *type, int len)
 	size_t linelen;
 	pid_t pid;
 	FILE *f;
-	char *sp1, *sp2, *sp3, *line = NULL;
-	char *srcdev;
+	char *sp1, *sp2, *sp3, *srcdev, *line = NULL;
 
 	if (!bdev || !bdev->src || !bdev->dest)
 		return -1;
 
-	srcdev = bdev->src;
-	if (strcmp(bdev->type, "loop") == 0)
-		srcdev = bdev->src + 5;
+	srcdev = lxc_storage_get_path(bdev->src, bdev->type);
 
 	ret = pipe(p);
 	if (ret < 0)
@@ -228,7 +218,7 @@ int detect_fs(struct lxc_storage *bdev, char *type, int len)
 		exit(1);
 	}
 
-	// if symlink, get the real dev name
+	/* if symlink, get the real dev name */
 	char devpath[MAXPATHLEN];
 	char *l = linkderef(srcdev, devpath);
 	if (!l)
@@ -288,7 +278,6 @@ int do_mkfs_exec_wrapper(void *args)
 	TRACE("executing \"%s %s\"", mkfs, data[1]);
 	execlp(mkfs, mkfs, data[1], (char *)NULL);
 	SYSERROR("failed to run \"%s %s \"", mkfs, data[1]);
-	free(mkfs);
 	return -1;
 }
 
@@ -426,11 +415,13 @@ bool unpriv_snap_allowed(struct lxc_storage *b, const char *t, bool snap,
 			 bool maybesnap)
 {
 	if (!t) {
-		// new type will be same as original
-		// (unless snap && b->type == dir, in which case it will be
-		// overlayfs -- which is also allowed)
+		/* New type will be same as original (unless snap && b->type ==
+		 * dir, in which case it will be overlayfs -- which is also
+		 * allowed).
+		 */
 		if (strcmp(b->type, "dir") == 0 ||
 		    strcmp(b->type, "aufs") == 0 ||
+		    strcmp(b->type, "overlay") == 0 ||
 		    strcmp(b->type, "overlayfs") == 0 ||
 		    strcmp(b->type, "btrfs") == 0 ||
 		    strcmp(b->type, "loop") == 0)
@@ -439,10 +430,12 @@ bool unpriv_snap_allowed(struct lxc_storage *b, const char *t, bool snap,
 		return false;
 	}
 
-	// unprivileged users can copy and snapshot dir, overlayfs,
-	// and loop.  In particular, not zfs, btrfs, or lvm.
+	/* Unprivileged users can copy and snapshot dir, overlayfs, and loop.
+	 * In particular, not zfs, btrfs, or lvm.
+	 */
 	if (strcmp(t, "dir") == 0 ||
 	    strcmp(t, "aufs") == 0 ||
+	    strcmp(t, "overlay") == 0 ||
 	    strcmp(t, "overlayfs") == 0 ||
 	    strcmp(t, "btrfs") == 0 ||
 	    strcmp(t, "loop") == 0)
@@ -459,6 +452,7 @@ bool is_valid_storage_type(const char *type)
 	    strcmp(type, "loop") == 0 ||
 	    strcmp(type, "lvm") == 0 ||
 	    strcmp(type, "nbd") == 0 ||
+	    strcmp(type, "overlay") == 0 ||
 	    strcmp(type, "overlayfs") == 0 ||
 	    strcmp(type, "rbd") == 0 ||
 	    strcmp(type, "zfs") == 0)
