@@ -77,7 +77,7 @@ static void usage(void) {
 
 int main(int argc, char *argv[])
 {
-	pid_t pid;
+	pid_t pid, sid;
 	int err;
 	char **aargv;
 	sigset_t mask, omask;
@@ -189,6 +189,10 @@ int main(int argc, char *argv[])
 			exit(EXIT_FAILURE);
 		}
 
+		sid = setsid();
+		if (sid < 0)
+			DEBUG("Failed to make child session leader");
+
 		NOTICE("About to exec '%s'", aargv[0]);
 
 		ret = execvp(aargv[0], aargv);
