@@ -241,6 +241,9 @@ int main(int argc, char *argv[])
 
 	const char *lxcpath = my_args.lxcpath[0];
 
+	/* REMOVE IN LXC 3.0 */
+	setenv("LXC_UPDATE_CONFIG_FORMAT", "1", 0);
+
 	/*
 	 * rcfile possibilities:
 	 * 1. rcfile from random path specified in cli option
