@@ -173,6 +173,9 @@ int main(int argc, char *argv[])
 		exit(ret);
 	lxc_log_options_no_override();
 
+	/* REMOVE IN LXC 3.0 */
+	setenv("LXC_UPDATE_CONFIG_FORMAT", "1", 0);
+
 	/* Set default timeout */
 	if (my_args.timeout == -2) {
 		if (my_args.hardstop)
