@@ -25,10 +25,9 @@
 
 #include <sys/types.h>
 #include <string.h>
-#include <unistd.h>
 #include <sys/stat.h>
-#include <sys/syscall.h>
 #include <errno.h>
+#include <unistd.h>
 
 #define TSTNAME    "lxc-attach-test"
 #define TSTOUT(fmt, ...) do { \
@@ -52,11 +51,11 @@ static void test_lsm_detect(void)
 {
 	if (lsm_enabled()) {
 		if (!strcmp(lsm_name(), "SELinux")) {
-			lsm_config_key = "lxc.se_context";
+			lsm_config_key = "lxc.selinux.context";
 			lsm_label      = "unconfined_u:unconfined_r:lxc_t:s0-s0:c0.c1023";
 		}
 		else if (!strcmp(lsm_name(), "AppArmor")) {
-			lsm_config_key = "lxc.aa_profile";
+			lsm_config_key = "lxc.apparmor.profile";
 			if (file_exists("/proc/self/ns/cgroup"))
 				lsm_label      = "lxc-container-default-cgns";
 			else
@@ -79,7 +78,7 @@ static void test_attach_lsm_set_config(struct lxc_container *ct)
 
 static int test_attach_lsm_func_func(void* payload)
 {
-	TSTOUT("%s", lsm_process_label_get(syscall(SYS_getpid)));
+	TSTOUT("%s", lsm_process_label_get(getpid()));
 	return 0;
 }
 
@@ -122,7 +121,7 @@ static int test_attach_lsm_func(struct lxc_container *ct)
 	ret = 0;
 
 err2:
-	(void)wait_for_pid(pid);
+	wait_for_pid(pid);
 err1:
 	close(pipefd[0]);
 	close(pipefd[1]);
@@ -176,7 +175,7 @@ static int test_attach_lsm_cmd(struct lxc_container *ct)
 	ret = 0;
 
 err2:
-	(void)wait_for_pid(pid);
+	wait_for_pid(pid);
 err1:
 	close(pipefd[0]);
 	close(pipefd[1]);
@@ -190,7 +189,7 @@ static int  test_attach_lsm_cmd(struct lxc_container *ct) { return 0; }
 
 static int test_attach_func_func(void* payload)
 {
-	TSTOUT("%d", (int)syscall(SYS_getpid));
+	TSTOUT("%d", getpid());
 	return 0;
 }
 
@@ -238,7 +237,7 @@ static int test_attach_func(struct lxc_container *ct)
 	ret = 0;
 
 err2:
-	(void)wait_for_pid(pid);
+	wait_for_pid(pid);
 err1:
 	close(pipefd[0]);
 	close(pipefd[1]);
@@ -404,6 +403,7 @@ int main(int argc, char *argv[])
 	if (ret < 0)
 		return EXIT_FAILURE;
 
+	(void)rmdir(LXCPATH "/alternate-path-test");
 	TSTOUT("All tests passed\n");
 	return EXIT_SUCCESS;
 }
